package NJ02;

use Exporter;
@ISA = (Exporter);
@EXPORT = qw(nj);

use Newick02;
    
sub nj{

my $ref_d_matrix = shift;

my @nodes;
my @d_taxa;
my @d_nodes;

# Creat distance matrix
my @d_matrix = @{$ref_d_matrix};
exit if $#d_matrix > 25;

my @species = ('A'..'Z');
splice (@species,$#d_matrix+1);

my $loop = $#d_matrix;
my $cycle = 0;
my $str1;
my $str2;

# Main loop for calculation
while ($loop > 0){
    my $N = $#d_matrix;
    # Calculate r
    my @r_array;
    foreach my $row (@d_matrix){
        my @d_array = @{$row};
        my $r;
        foreach my $d (@d_array){
            $r += $d;
        }
        push @r_array, $r;
    }
    
    # If last loop, join the remaining taxa with the top node
    if ($loop == 1){
        my $fd =sprintf("%.3f",$r_array[0]);
        push @nodes, 'X';
        push @d_taxa, [$species[0].':'.$fd, $species[1].':'.$fd];
        $str1 .=  $species[0]." -<".$fd.">- ". $species[1];
        last;
    }
    
    # Calculate M matrix
    my @M_matrix;
    for (my $i=0; $i<=$N; $i++){
        for (my $j=0; $j<=$N; $j++){
            if ($i==$j){
                $M_matrix[$i][$j] = 0;
            }else{
                $M_matrix[$i][$j] = $d_matrix[$i][$j]-($r_array[$i]+$r_array[$j])/($N+1-2);
            }
        }
    }
        
    # Find minimum value in M matrix (found first)
    my $min_M = 0;
    my $min_i = -1;
    my $min_j = -1;
    for (my $i=0; $i<=$N; $i++){
        for (my $j=0; $j<=$N; $j++){
                last if ($j >= $i);
                if ($M_matrix[$i][$j] < $min_M){
                    $min_M = $M_matrix[$i][$j];
                    $min_i = $i;
                    $min_j = $j;
                }
        }
    }
    
    # Calculate S
    my $s1 = $d_matrix[$min_i][$min_j]/2 + ($r_array[$min_i]-$r_array[$min_j])/(2*($N+1-2));
    my $s2 = $d_matrix[$min_i][$min_j]-$s1;
    
    my $esc_spec = $species[$min_j]; # Just for print out
    $species[$min_j] = 'U'.$cycle; # Replace one taxa with a node

    # ----- Print out result 1  ----- #  
    # Store tree guide
    my $fs1=sprintf("%.3f",$s1);
    my $fs2=sprintf("%.3f",$s2);
    
    push @nodes, $species[$min_j];
    push @d_taxa, [$species[$min_i].':'.$fs1, $esc_spec.':'.$fs2];

    $str1 .= $species[$min_i]." -<".$fs1.">- \n";
    $str1 .= "       |\n";
    $str1 .= "       $species[$min_j]\n";
    $str1 .= "       |\n";
    $str1 .= $esc_spec." -<".$fs2.">- \n\n";

    # Calculate distance from min(i,j) -> each node
    # and create a new matrix
    my $row = 0;
    my $col = 0;
    my @new_matrix;
    
    for (my $i=0; $i<=$N; $i++){
        next if ($i == $min_i);
        for (my $j=0; $j<=$N; $j++){
            if ($i == $min_i || $j == $min_i){
                #$d_matrix[$i][$j] = 0;
            }elsif($i == $min_j){ # || 
                $new_matrix[$row][$col] = ($d_matrix[$min_i][$j]+$d_matrix[$min_j][$j]-$d_matrix[$min_i][$min_j])/2;
                $col++;
            }elsif($j == $min_j){
                $new_matrix[$row][$col] = ($d_matrix[$i][$min_i]+$d_matrix[$i][$min_j]-$d_matrix[$min_i][$min_j])/2;
                $col++;
            }else{
                $new_matrix[$row][$col] =  $d_matrix[$i][$j];
                $col++;
            }
        }
        $col = 0;
        $row++;
    }

    undef @d_matrix;
    @d_matrix = @new_matrix;
    
    splice (@species, $min_i, 1); # Remove the other taxa
    
    $loop--;
    $cycle++;
}



my @nodes2;
my @d_taxa2;

foreach (@d_taxa){
    my @array = @{$_};
    foreach (@array){
        if ($_=~/^(U\d):(.+)/){
            push @nodes2, $1;
            push @d_nodes,$2;
        }
    }
}
print "\n";

for (my $i=0;$i<=$#d_taxa;$i++){
    my @array = @{$d_taxa[$i]};
    for (my $j=0;$j<=$#array;$j++){
        if ($d_taxa[$i][$j]=~/^(U\d):(.+)/){
            $d_taxa2[$i][$j] = $1;
        }else{
            $d_taxa2[$i][$j] = $d_taxa[$i][$j];
        }
    }
}

$str2 = &newick(\@nodes,\@nodes2,\@d_taxa2,\@d_nodes);

return ($str1,$str2);

}

return 1;

__END__

print "nodes\n";
print "$_," foreach @nodes;
print "\n\n";

print "d_taxa\n";
    print "$_," foreach @array;
    print "\n";

print "nodes2\n";
print "$_," foreach @nodes2;
print "\n\n";


print "d_nodes\n";
print "$_," foreach @d_nodes;
print "\n\n";

print "d_taxa2\n";
foreach (@d_taxa2){
    my @array = @{$_};
    print "$_," foreach @array;
    print "\n";
}

# ----- Print out result 1  ----- #  
print "$species[$min_i] = $s1, $esc_spec = $s2\n";
print $species[$min_j]." = ".$species[$min_i]."+".$esc_spec;
print "\n\n";

